@@ -1,3 +1,19 @@
+# Copyright (C) 2024
+# Wassim Jabi <wassim.jabi@gmail.com>
+#
+# This program is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Affero General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Affero General Public License along with
+# this program. If not, see <https://www.gnu.org/licenses/>.
+
 import topologic
 from topologic import IntAttribute, DoubleAttribute, StringAttribute, ListAttribute
 
@@ -30,6 +46,74 @@ class Dictionary(topologic.Dictionary):
             dictionaries.append(Dictionary.ByKeysValues(keys, values))
         return dictionaries
     '''
+    @staticmethod
+    def ByKeyValue(key, value):
+        """
+        Creates a Dictionary from the input key and the input value.
+
+        Parameters
+        ----------
+        key : str
+            The string representing the key of the value in the dictionary.
+        value : int, float, str, or list
+            A value corresponding to the input key. A value can be an integer, a float, a string, or a list.
+
+        Returns
+        -------
+        topologic.Dictionary
+            The created dictionary.
+
+        """
+        if not isinstance(key, str):
+            print("Dictionary.ByKeyValue - Error: The input key is not a valid string. Returning None.")
+            return None
+        return Dictionary.ByKeysValues([key], [value])
+    
+    
+    @staticmethod
+    def _ConvertValue(value):
+        """
+        Converts the input value to the proper attribute
+        """
+        from topologicpy.Topology import Topology
+        import json
+
+        def dict_to_json(py_dict):
+            """
+            Convert a Python dictionary to a JSON-formatted string.
+            """
+            return json.dumps(py_dict, indent=2)
+        
+        attr = topologic.StringAttribute("__NONE__")
+        if value == None:
+            attr = topologic.StringAttribute("__NONE__")
+        elif isinstance(value, bool):
+            if value == False:
+                attr = topologic.IntAttribute(0)
+            else:
+                attr = topologic.IntAttribute(1)
+        elif isinstance(value, int):
+            attr = topologic.IntAttribute(value)
+        elif isinstance(value, float):
+            attr = topologic.DoubleAttribute(value)
+        elif isinstance(value, topologic.Topology):
+            str_value = Topology.JSONString(value)
+            attr = topologic.StringAttribute(str_value)
+        elif isinstance(value, dict):
+            str_value = dict_to_json(value)
+            attr = topologic.StringAttribute(str_value)
+        elif isinstance(value, str):
+            attr = topologic.StringAttribute(value)
+        elif isinstance(value, tuple):
+            l = [Dictionary._ConvertValue(v) for v in list(value)]
+            attr = topologic.ListAttribute(l)
+        elif isinstance(value, list):
+            l = [Dictionary._ConvertValue(v) for v in value]
+            attr = topologic.ListAttribute(l)
+        else:
+            attr = topologic.StringAttribute("__NONE__")
+        return attr
+    
     @staticmethod
     def ByKeysValues(keys, values):
         """
@@ -48,9 +132,15 @@ class Dictionary(topologic.Dictionary):
             The created dictionary.
 
         """
-        if not isinstance(keys, list) or not isinstance(values, list):
+        
+        if not isinstance(keys, list):
+            print("Dictionary.ByKeysValues - Error: The input keys parameter is not a valid list. Returning None.")
+            return None
+        if not isinstance(values, list):
+            print("Dictionary.ByKeysValues - Error: The input values parameter is not a valid list. Returning None.")
             return None
         if len(keys) != len(values):
+            print("Dictionary.ByKeysValues - Error: The input keys and values parameters are not of equal length. Returning None.")
             return None
         stl_keys = []
         stl_values = []
@@ -59,35 +149,7 @@ class Dictionary(topologic.Dictionary):
                 stl_keys.append(keys[i])
             else:
                 stl_keys.append(str(keys[i]))
-            if isinstance(values[i], list) and len(values[i]) == 1:
-                value = values[i][0]
-            else:
-                value = values[i]
-            if isinstance(value, bool):
-                if value == False:
-                    stl_values.append(topologic.IntAttribute(0))
-                else:
-                    stl_values.append(topologic.IntAttribute(1))
-            elif isinstance(value, int):
-                stl_values.append(topologic.IntAttribute(value))
-            elif isinstance(value, float):
-                stl_values.append(topologic.DoubleAttribute(value))
-            elif isinstance(value, str):
-                stl_values.append(topologic.StringAttribute(value))
-            elif isinstance(value, list):
-                l = []
-                for v in value:
-                    if isinstance(v, bool):
-                        l.append(topologic.IntAttribute(v))
-                    elif isinstance(v, int):
-                        l.append(topologic.IntAttribute(v))
-                    elif isinstance(v, float):
-                        l.append(topologic.DoubleAttribute(v))
-                    elif isinstance(v, str):
-                        l.append(topologic.StringAttribute(v))
-                stl_values.append(topologic.ListAttribute(l))
-            else:
-                return None
+            stl_values.append(Dictionary._ConvertValue(values[i]))
         return topologic.Dictionary.ByKeysValues(stl_keys, stl_values)
     
     @staticmethod
@@ -102,25 +164,42 @@ class Dictionary(topologic.Dictionary):
 
         Returns
         -------
-        topologic.DIctionary
+        topologic.Dictionary
             The created dictionary.
 
         """
+        if not isinstance(dictionaries, list):
+            print("Dictionary.ByMergedDictionaries - Error: The input dictionaries parameter is not a valid list. Returning None.")
+            return None
+        new_dictionaries = []
+        for d in dictionaries:
+            if isinstance(d, topologic.Dictionary):
+                new_dictionaries.append(d)
+            elif isinstance(d, dict):
+                new_dictionaries.append(Dictionary.ByPythonDictionary(d))
+        if len(new_dictionaries) == 0:
+            print("Dictionary.ByMergedDictionaries - Error: The input dictionaries parameter does not contain valid dictionaries. Returning None.")
+            return None
+        elif len(new_dictionaries) == 1:
+            print("Dictionary.ByMergedDictionaries - Warning: The input dictionaries parameter contains only one valid dictionary. Returning that dictionary.")
+            return new_dictionaries[0]
+        else:
+            dictionaries = new_dictionaries
         sinkKeys = []
         sinkValues = []
         d = dictionaries[0]
         if d != None:
             stlKeys = d.Keys()
             if len(stlKeys) > 0:
-                sinkKeys = d.Keys()
+                sinkKeys = Dictionary.Keys(d)
                 sinkValues = Dictionary.Values(d)
             for i in range(1,len(dictionaries)):
                 d = dictionaries[i]
                 if d == None:
                     continue
-                stlKeys = d.Keys()
+                stlKeys = Dictionary.Keys(d)
                 if len(stlKeys) > 0:
-                    sourceKeys = d.Keys()
+                    sourceKeys = Dictionary.Keys(d)
                     for aSourceKey in sourceKeys:
                         if aSourceKey not in sinkKeys:
                             sinkKeys.append(aSourceKey)
@@ -131,9 +210,11 @@ class Dictionary(topologic.Dictionary):
                         if sourceValue != None:
                             if sinkValues[index] != "":
                                 if isinstance(sinkValues[index], list):
-                                    sinkValues[index].append(sourceValue)
+                                    if not sourceValue in sinkValues[index]:
+                                        sinkValues[index].append(sourceValue)
                                 else:
-                                    sinkValues[index] = [sinkValues[index], sourceValue]
+                                    if not sourceValue == sinkValues[index]:
+                                        sinkValues[index] = [sinkValues[index], sourceValue]
                             else:
                                 sinkValues[index] = sourceValue
         if len(sinkKeys) > 0 and len(sinkValues) > 0:
@@ -235,6 +316,7 @@ class Dictionary(topologic.Dictionary):
 
         """
         if not isinstance(pythonDictionary, dict):
+            print("Dictionary.ByPythonDictionary - Error: The input dictionary parameter is not a valid python dictionary. Returning None.")
             return None
         keys = list(pythonDictionary.keys())
         values = []
@@ -258,6 +340,9 @@ class Dictionary(topologic.Dictionary):
             The list of keys of the input dictionary.
 
         """
+        if not isinstance(dictionary, topologic.Dictionary) and not isinstance(dictionary, dict):
+            print("Dictionary.Keys - Error: The input dictionary parameter is not a valid topologic or python dictionary. Returning None.")
+            return None
         if isinstance(dictionary, dict):
             return list(dictionary.keys())
         elif isinstance(dictionary, topologic.Dictionary):
@@ -282,16 +367,7 @@ class Dictionary(topologic.Dictionary):
 
         """
         listAttributes = listAttribute.ListValue()
-        returnList = []
-        for attr in listAttributes:
-            if isinstance(attr, IntAttribute):
-                returnList.append(attr.IntValue())
-            elif isinstance(attr, DoubleAttribute):
-                returnList.append(attr.DoubleValue())
-            elif isinstance(attr, StringAttribute):
-                returnList.append(attr.StringValue())
-            elif isinstance(attr, float) or isinstance(attr, int) or isinstance(attr, str) or isinstance(attr, dict):
-                returnList.append(attr)
+        returnList = [Dictionary._ConvertAttribute(attr) for attr in listAttributes]
         return returnList    
        
     @staticmethod
@@ -301,7 +377,7 @@ class Dictionary(topologic.Dictionary):
 
         Parameters
         ----------
-        dicitonary : topologic.Dictionary
+        dictionary : topologic.Dictionary
             The input dictionary.
 
         Returns
@@ -310,7 +386,11 @@ class Dictionary(topologic.Dictionary):
             The python dictionary equivalent of the input dictionary
 
         """
+        if isinstance(dictionary, dict):
+            print("Dictionary.PythonDictionary - Warning: The input dictionary parameter is already a python dictionary. Returning that dictionary.")
+            return dictionary
         if not isinstance(dictionary, topologic.Dictionary):
+            print("Dictionary.PythonDictionary - Error: The input dictionary parameter is not a valid topologic dictionary. Returning None.")
             return None
         keys = dictionary.Keys()
         pythonDict = {}
@@ -324,7 +404,11 @@ class Dictionary(topologic.Dictionary):
             elif isinstance(attr, topologic.DoubleAttribute):
                 pythonDict[key] = (attr.DoubleValue())
             elif isinstance(attr, topologic.StringAttribute):
-                pythonDict[key] = (attr.StringValue())
+                temp_str = attr.StringValue()
+                if temp_str == "__NONE__":
+                    pythonDict[key] = None
+                else:
+                    pythonDict[key] = (temp_str)
             elif isinstance(attr, topologic.ListAttribute):
                 pythonDict[key] = (Dictionary.ListAttributeValues(attr))
             else:
@@ -338,7 +422,7 @@ class Dictionary(topologic.Dictionary):
 
         Parameters
         ----------
-        dicitonary : topologic.Dictionary or dict
+        dictionary : topologic.Dictionary or dict
             The input dictionary.
         key : string
             The input key.
@@ -352,6 +436,8 @@ class Dictionary(topologic.Dictionary):
 
         """
         def processPythonDictionary (dictionary, key, value):
+            if value == "__NONE__":
+                value = None
             dictionary[key] = value
             return dictionary
 
@@ -365,8 +451,17 @@ class Dictionary(topologic.Dictionary):
                     values.append(value)
                 else:
                     values.append(Dictionary.ValueAtKey(dictionary, k))
-            return Dictionary.ByKeysValues(keys, values)
-
+            d = Dictionary.ByKeysValues(keys, values)
+            return d
+        
+        if not isinstance(dictionary, topologic.Dictionary) and not isinstance(dictionary, dict):
+            print("Dictionary.SetValueAtKey - Error: The input dictionary parameter is not a valid topologic or python dictionary. Returning None.")
+            return None
+        if not isinstance(key, str):
+            print("Dictionary.SetValueAtKey - Error: The input key parameter is not a valid string. Returning None.")
+            return None
+        if value == None:
+            value = "__NONE__"
         if isinstance(dictionary, dict):
             return processPythonDictionary(dictionary, key, value)
         elif isinstance(dictionary, topologic.Dictionary):
@@ -374,6 +469,83 @@ class Dictionary(topologic.Dictionary):
         else:
             return None
  
+    @staticmethod
+    def _ConvertAttribute(attr):
+        """
+            Convert the found attribute into the proper value
+        """
+        from topologicpy.Topology import Topology
+        import json
+
+        def is_json_string(input_string):
+            """
+            Check if the input string is a valid JSON string.
+            """
+            try:
+                json.loads(input_string)
+                return True
+            except json.JSONDecodeError:
+                return False
+
+        def json_to_dict(json_string):
+            """
+            Convert a JSON-formatted string to a Python dictionary.
+            """
+            return json.loads(json_string)
+        
+        if isinstance(attr, IntAttribute):
+            return (attr.IntValue())
+        elif isinstance(attr, DoubleAttribute):
+            return (attr.DoubleValue())
+        elif isinstance(attr, StringAttribute):
+            temp_value = attr.StringValue()
+            topologies = None
+            try:
+                topologies = Topology.ByJSONString(temp_value, progressBar=False)
+            except:
+                topologies = None
+            if isinstance(topologies, list):
+                if len(topologies) == 0:
+                    topologies = None
+            if temp_value == "__NONE__":
+                return None
+            elif isinstance(topologies, topologic.Topology):
+                return topologies
+            elif isinstance(topologies, list):
+                if len(topologies) > 1:
+                    return topologies
+                elif len(topologies) == 1:
+                    return topologies[0]
+            elif is_json_string(temp_value):
+                ret_value = json_to_dict(temp_value)
+                return ret_value
+            else:
+                return (temp_value)
+        elif isinstance(attr, ListAttribute):
+            return (Dictionary.ListAttributeValues(attr))
+        elif isinstance(attr, float) or isinstance(attr, int):
+            return attr
+        elif isinstance(attr, str):
+            topologies = Topology.ByJSONString(attr)
+            if attr == "__NONE__":
+                return None
+            elif len(topologies) > 1:
+                return topologies
+            elif len(topologies) == 1:
+                return topologies[0]
+            elif is_json_string(attr):
+                return json_to_dict(attr)
+            else:
+                return (attr)
+        elif isinstance(attr, tuple):
+            return Dictionary.ListAttributeValues([Dictionary._ConvertAttribute(x) for x in list(attr)])
+        elif isinstance(attr, list):
+            return Dictionary.ListAttributeValues([Dictionary._ConvertAttribute(x) for x in attr])
+        elif isinstance(attr, dict):
+            return attr
+        else:
+            return None
+    
     @staticmethod
     def ValueAtKey(dictionary, key):
         """
@@ -392,29 +564,21 @@ class Dictionary(topologic.Dictionary):
             The value found at the input key in the input dictionary.
 
         """
+        import json
+        from topologicpy.Topology import Topology
+        
+        
+        if not isinstance(dictionary, topologic.Dictionary) and not isinstance(dictionary, dict):
+            print("Dictionary.ValueAtKey - Error: The input dictionary parameter is not a valid topologic or python dictionary. Returning None.")
+            return None
         if isinstance(dictionary, dict):
             attr = dictionary[key]
         elif isinstance(dictionary, topologic.Dictionary):
             attr = dictionary.ValueAtKey(key)
         else:
             return None
-        
-        if isinstance(attr, IntAttribute):
-            return (attr.IntValue())
-        elif isinstance(attr, DoubleAttribute):
-            return (attr.DoubleValue())
-        elif isinstance(attr, StringAttribute):
-            return (attr.StringValue())
-        elif isinstance(attr, ListAttribute):
-            return (Dictionary.ListAttributeValues(attr))
-        elif isinstance(attr, float) or isinstance(attr, int) or isinstance(attr, str):
-            return attr
-        elif isinstance(attr, list):
-            return Dictionary.ListAttributeValues(attr)
-        elif isinstance(attr, dict):
-            return attr
-        else:
-            return None
+        return_value = Dictionary._ConvertAttribute(attr)
+        return return_value
         
     @staticmethod
     def Values(dictionary):
@@ -432,6 +596,9 @@ class Dictionary(topologic.Dictionary):
             The list of values found in the input dictionary.
 
         """
+        if not isinstance(dictionary, topologic.Dictionary) and not isinstance(dictionary, dict):
+            print("Dictionary.Values - Error: The input dictionary parameter is not a valid topologic or python dictionary. Returning None.")
+            return None
         keys = None
         if isinstance(dictionary, dict):
             keys = dictionary.keys()
@@ -445,17 +612,23 @@ class Dictionary(topologic.Dictionary):
                 if isinstance(dictionary, dict):
                     attr = dictionary[key]
                 elif isinstance(dictionary, topologic.Dictionary):
-                    attr = dictionary.ValueAtKey(key)
+                    attr = Dictionary.ValueAtKey(dictionary,key)
                 else:
                     attr = None
             except:
                 return None
+            returnList.append(attr)
+            '''
             if isinstance(attr, topologic.IntAttribute):
                 returnList.append(attr.IntValue())
             elif isinstance(attr, topologic.DoubleAttribute):
                 returnList.append(attr.DoubleValue())
             elif isinstance(attr, topologic.StringAttribute):
-                returnList.append(attr.StringValue())
+                temp_attr = attr.StringValue()
+                if temp_attr == "__NONE__":
+                    returnList.append(None)
+                else:
+                    returnList.append(attr.StringValue())
             elif isinstance(attr, topologic.ListAttribute):
                 returnList.append(Dictionary.ListAttributeValues(attr))
             elif isinstance(attr, float) or isinstance(attr, int) or isinstance(attr, str):
@@ -464,6 +637,7 @@ class Dictionary(topologic.Dictionary):
                 returnList.append(Dictionary.ListAttributeValues(attr))
             else:
                 returnList.append("")
+            '''
         return returnList
     
     
