@@ -1,24 +1,114 @@
-import honeybee_energy.lib.constructionsets as constr_set_lib
-import honeybee.facetype
-from honeybee.face import Face as HBFace
-from ladybug_geometry.geometry3d.face import Face3D
-from honeybee.model import Model as HBModel
-from honeybee.room import Room as HBRoom
-from honeybee.shade import Shade as HBShade
-from honeybee.aperture import Aperture as HBAperture
-from honeybee.door import Door as HBDoor
-from honeybee_energy.schedule.ruleset import ScheduleRuleset
-from honeybee_energy.schedule.day import ScheduleDay
-from honeybee_energy.load.setpoint import Setpoint
-from honeybee_energy.load.hotwater import  ServiceHotWater
-import honeybee_energy.lib.programtypes as prog_type_lib
-import honeybee_energy.lib.scheduletypelimits as schedule_types
-from honeybee_radiance.sensorgrid import SensorGrid
+# Copyright (C) 2024
+# Wassim Jabi <wassim.jabi@gmail.com>
+#
+# This program is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Affero General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Affero General Public License along with
+# this program. If not, see <https://www.gnu.org/licenses/>.
+
+import os
+import warnings
+
+try:
+    import honeybee.facetype
+    from honeybee.face import Face as HBFace
+    from honeybee.model import Model as HBModel
+    from honeybee.room import Room as HBRoom
+    from honeybee.shade import Shade as HBShade
+    from honeybee.aperture import Aperture as HBAperture
+    from honeybee.door import Door as HBDoor
+except:
+    print("Honeybee - Installing required honeybee library.")
+    try:
+        os.system("pip install honeybee")
+    except:
+        os.system("pip install honeybee --user")
+    try:
+        import honeybee.facetype
+        from honeybee.face import Face as HBFace
+        from honeybee.model import Model as HBModel
+        from honeybee.room import Room as HBRoom
+        from honeybee.shade import Shade as HBShade
+        from honeybee.aperture import Aperture as HBAperture
+        from honeybee.door import Door as HBDoor
+    except:
+        warnings.warn("Honeybee - ERROR: Could not import honeybee")
+
+try:
+    import honeybee_energy.lib.constructionsets as constr_set_lib
+    import honeybee_energy.lib.programtypes as prog_type_lib
+    import honeybee_energy.lib.scheduletypelimits as schedule_types
+    from honeybee_energy.schedule.ruleset import ScheduleRuleset
+    from honeybee_energy.schedule.day import ScheduleDay
+    from honeybee_energy.load.setpoint import Setpoint
+    from honeybee_energy.load.hotwater import  ServiceHotWater
+except:
+    print("Honeybee - Installing required honeybee-energy library.")
+    try:
+        os.system("pip install -U honeybee-energy[standards]")
+    except:
+        os.system("pip install -U honeybee-energy[standards] --user")
+    try:
+        import honeybee_energy.lib.constructionsets as constr_set_lib
+        import honeybee_energy.lib.programtypes as prog_type_lib
+        import honeybee_energy.lib.scheduletypelimits as schedule_types
+        from honeybee_energy.schedule.ruleset import ScheduleRuleset
+        from honeybee_energy.schedule.day import ScheduleDay
+        from honeybee_energy.load.setpoint import Setpoint
+        from honeybee_energy.load.hotwater import  ServiceHotWater
+    except:
+        warnings.warn("Honeybee - Error: Could not import honeybee-energy")
+
+try:
+    from honeybee_radiance.sensorgrid import SensorGrid
+except:
+    print("Honeybee - Installing required honeybee-radiance library.")
+    try:
+        os.system("pip install -U honeybee-radiance")
+    except:
+        os.system("pip install -U honeybee-radiance --user")
+    try:
+        from honeybee_radiance.sensorgrid import SensorGrid
+    except:
+        warnings.warn("Honeybee - Error: Could not import honeybee-radiance")
+
+try:
+    from ladybug.dt import Time
+except:
+    print("Honeybee - Installing required ladybug library.")
+    try:
+        os.system("pip install -U ladybug")
+    except:
+        os.system("pip install -U ladybug --user")
+    try:
+        from ladybug.dt import Time
+    except:
+        warnings.warn("Honeybee - Error: Could not import ladybug")
+
+try:
+    from ladybug_geometry.geometry3d.face import Face3D
+    from ladybug_geometry.geometry3d.pointvector import Point3D, Vector3D
+except:
+    print("Honeybee - Installing required ladybug-geometry library.")
+    try:
+        os.system("pip install -U ladybug-geometry")
+    except:
+        os.system("pip install -U ladybug-geometry --user")
+    try:
+        from ladybug_geometry.geometry3d.face import Face3D
+        from ladybug_geometry.geometry3d.pointvector import Point3D, Vector3D
+    except:
+        warnings.warn("Honeybee - Error: Could not import ladybug-geometry")
 
-from ladybug.dt import Time
-from ladybug_geometry.geometry3d.pointvector import Point3D, Vector3D
 import json
-from topologicpy.Dictionary import Dictionary
 import topologic
 
 class Honeybee:
@@ -58,7 +148,7 @@ class Honeybee:
         return [constrSets, constrIdentifiers]
     
     @staticmethod
-    def ExportToHBJSON(model, path, overwrite=True):
+    def ExportToHBJSON(model, path, overwrite=False):
         """
         Exports the input HB Model to a file.
 
@@ -69,7 +159,7 @@ class Honeybee:
         path : str
             The location of the output file.
         overwrite : bool , optional
-            If set to True this method overwrites any existing file. Otherwise, it won't. The default is True.
+            If set to True this method overwrites any existing file. Otherwise, it won't. The default is False.
 
         Returns
         -------
@@ -77,11 +167,16 @@ class Honeybee:
             Returns True if the operation is successful. Returns False otherwise.
 
         """
-        # hbModel, path = item
+        from os.path import exists
+
         # Make sure the file extension is .hbjson
         ext = path[len(path)-7:len(path)]
         if ext.lower() != ".hbjson":
             path = path+".hbjson"
+        
+        if not overwrite and exists(path):
+            print("DGL.ExportToHBJSON - Error: a file already exists at the specified path and overwrite is set to False. Returning None.")
+            return None
         f = None
         try:
             if overwrite == True:
@@ -89,7 +184,8 @@ class Honeybee:
             else:
                 f = open(path, "x") # Try to create a new File
         except:
-            raise Exception("Error: Could not create a new file at the following location: "+path)
+            print("DGL.ExportToHBJSON - Error: Could not create a new file at the following location: "+path+". Returning None.")
+            return None
         if (f):
             json.dump(model.to_dict(), f, indent=4)
             f.close()    
@@ -98,7 +194,7 @@ class Honeybee:
     
     @staticmethod
     def ModelByTopology(tpBuilding,
-                tpShadingFacesCluster=None,
+                tpShadingFacesCluster = None,
                 buildingName = "Generic_Building",
                 defaultProgramIdentifier = "Generic Office Program",
                 defaultConstructionSetIdentifier = "Default Generic Construction Set",
@@ -106,12 +202,18 @@ class Honeybee:
                 heatingSetpoint = 20.0,
                 humidifyingSetpoint = 30.0,
                 dehumidifyingSetpoint = 55.0,
-                roomNameKey = "name",
-                roomTypeKey = "type",
-                apertureTypeKey = "type",
+                roomNameKey = "TOPOLOGIC_name",
+                roomTypeKey = "TOPOLOGIC_type",
+                apertureTypeKey = "TOPOLOGIC_type",
                 addSensorGrid = False):
         """
-        Creates an HB Model from the input Topology
+        Creates an HB Model from the input Topology.
+
+        Parameters
+        ----------
+
+
+        Returns
         -------
         HBModel
             The created HB Model
@@ -145,7 +247,10 @@ class Honeybee:
             return returnList
 
         def getKeyName(d, keyName):
-            keys = Dictionary.Keys(d)
+            if not d == None:
+                keys = Dictionary.Keys(d)
+            else:
+                keys = []
             for key in keys:
                 if key.lower() == keyName.lower():
                     return key
